// (C) Copyright 2002-2007 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
// Cyrille Fauvel
// Autodesk Developer Network

//-------------------------------------------------------------------------
//- asdkLongTransactionReactor.cpp : Implementation of asdkLongTransactionReactor
//-------------------------------------------------------------------------
#include "StdAfx.h"
#include "asdkLongTransactionReactor.h"

#include "../OPLine/asdkOPLineReactor.h"

//-------------------------------------------------------------------------
ACRX_CONS_DEFINE_MEMBERS(asdkLongTransactionReactor, AcApLongTransactionReactor, 1)

//-------------------------------------------------------------------------
asdkLongTransactionReactor::asdkLongTransactionReactor (const bool autoInitAndRelease) : AcApLongTransactionReactor(), mbAutoInitAndRelease(autoInitAndRelease) {
	if ( autoInitAndRelease ) {
		if ( acapLongTransactionManager )
			acapLongTransactionManager->addReactor (this) ;
		else
			mbAutoInitAndRelease =false ;
	}
}

//-------------------------------------------------------------------------
asdkLongTransactionReactor::~asdkLongTransactionReactor () {
	Detach () ;
}

//-------------------------------------------------------------------------
void asdkLongTransactionReactor::Attach () {
	Detach () ;
	if ( !mbAutoInitAndRelease ) {
		if ( acapLongTransactionManager ) {
			acapLongTransactionManager->addReactor (this) ;
			mbAutoInitAndRelease =true ;
		}
	}
}

void asdkLongTransactionReactor::Detach () {
	if ( mbAutoInitAndRelease ) {
		if ( acapLongTransactionManager ) {
			acapLongTransactionManager->removeReactor (this) ;
			mbAutoInitAndRelease =false ;
		}
	}
}

AcApLongTransactionManager *asdkLongTransactionReactor::Subject () const {
	return (acapLongTransactionManager) ;
}

bool asdkLongTransactionReactor::IsAttached () const {
	return (mbAutoInitAndRelease) ;
}

//-------------------------------------------------------------------------
void asdkLongTransactionReactor::beginCheckOut (AcDbLongTransaction &longTr, AcDbObjectIdArray &originList) {
	for ( int j =0 ; j < originList.length () ; j++ ) {
		AcDbObjectPointer<AcDbPolyline> pLine (originList.at (j), AcDb::kForRead) ;
		if ( pLine.openStatus () != Acad::eOk )
			continue ;

		AcDbVoidPtrArray *ptr =pLine->reactors () ;
		for ( int i =0 ; ptr != NULL && i < ptr->length () ; i++ ) {
			if ( acdbIsPersistentReactor (ptr->at (i)) ) {
				//- May be one of our reactor is in?
				AcDbObjectId idr =acdbPersistentReactorObjectId (ptr->at (i)) ;
				AcDbObjectPointer<asdkOPLineReactor> pReac (idr, AcDb::kForRead) ;
				if ( pReac.openStatus () != Acad::eOk )
					continue ;

				//- Main entity
				AcDbDictionaryPointer pExtDct (pReac->ownerId (), AcDb::kForRead) ;
				if ( pExtDct.openStatus () != Acad::eOk )
					continue ;
				AcDbObjectId mainEntId =pExtDct->ownerId () ;

				if ( !originList.contains (mainEntId) ) {
					originList.append (mainEntId) ;
					longTr.addToWorkSet (mainEntId) ;
				}

				for ( int k =0 ; k < pReac->entsToUpdate ().length () ; k++ ) {
					if ( !originList.contains (pReac->entsToUpdate ().at (k)) ) {
						originList.append (pReac->entsToUpdate ().at (k)) ;
						longTr.addToWorkSet (pReac->entsToUpdate ().at (k)) ;
					}
				}
				break ;
			}
		}
		//- Do not do the following...
		//delete [] ptr ;
	}
}

void asdkLongTransactionReactor::endCheckIn (AcDbLongTransaction &longTr) {
	AcDbIdMapping &idMap =*longTr.activeIdMap () ;
	AcDbDatabase *pOrigDb =NULL, *pDestDb =NULL ;
	idMap.origDb (pOrigDb) ;
	idMap.destDb (pDestDb) ;

	AcDbIdMappingIter iter (idMap) ;
	AcDbIdPair idPair ;
	for ( iter.start () ; !iter.done () ; iter.next () ) {
		if ( !iter.getMap (idPair) )
			continue ;
		if ( !idPair.isCloned () )
			continue ;
		AcDbObjectId idOrig =longTr.originObject (idPair.key ()) ;
		if ( idOrig.isNull () )
			continue ; //- Newly added objects need not be fixed
		AcDbObjectPointer<asdkOPLineReactor> pO (idPair.value (), AcDb::kForRead) ;
		if ( pO.openStatus () == Acad::eOk ) {
			//- Main entity
			AcDbDictionaryPointer pDict (pO->ownerId (), AcDb::kForRead) ;
			if ( pDict.openStatus () == Acad::eOk ) {
				AcDbObjectPointer<AcDbEntity> pEnt (pDict->ownerId (), AcDb::kForWrite) ;
				if ( pEnt.openStatus () == Acad::eOk ) {
					//- Make sure that the reactors are fired on the right object
					AcDbIdMapping idFixMap ;
					idFixMap.setDestDb (pDestDb) ;
					AcDbIdPair idFixPair ;
					idFixPair.setKey (idOrig) ;
					idFixPair.setValue (idPair.value ()) ;
					idFixMap.assign (idFixPair) ;
					pEnt->swapReferences (idFixMap) ;
				}
			}
			//- Sub entity(ies)
			for ( int i =0 ; i < pO->entsToUpdate ().length () ; i++ ) {
				AcDbObjectPointer<AcDbEntity> pEnt (pO->entsToUpdate ().at (i), AcDb::kForWrite) ;
				if ( pEnt.openStatus () == Acad::eOk ) {
					//- Make sure that the reactors are fired on the right object
					AcDbIdMapping idFixMap ;
					idFixMap.setDestDb (pDestDb) ;
					AcDbIdPair idFixPair ;
					idFixPair.setKey (idOrig) ;
					idFixPair.setValue (idPair.value ()) ;
					idFixMap.assign (idFixPair) ;
					pEnt->swapReferences (idFixMap) ;
				}
			}
		}
	}
}